package benefactorhandler

import (
	"net/http"

	param "git.gocasts.ir/ebhomengo/niki/param/benefactor/address"
	"git.gocasts.ir/ebhomengo/niki/pkg/claim"
	httpmsg "git.gocasts.ir/ebhomengo/niki/pkg/http_msg"
	"github.com/labstack/echo/v4"
)

func (h Handler) AddAddress(c echo.Context) error {
	claims := claim.GetClaimsFromEchoContext(c)
	req := param.BenefactorAddAddressRequest{BenefactorID: claims.UserID}
	if bErr := c.Bind(&req); bErr != nil {
		return echo.NewHTTPError(http.StatusBadRequest)
	}

	if fieldErrors, err := h.benefactorVld.ValidateAddAddress(req); err != nil {
		msg, code := httpmsg.Error(err)

		return c.JSON(code, echo.Map{
			"message": msg,
			"errors":  fieldErrors,
		})
	}
	resp, sErr := h.benefactorAddressSvc.Add(c.Request().Context(), req)

	if sErr != nil {
		msg, code := httpmsg.Error(sErr)

		return echo.NewHTTPError(code, msg)
	}

	return c.JSON(http.StatusCreated, resp)
}
