package adminkindboxvalidator

import (
	"errors"

	param "git.gocasts.ir/ebhomengo/niki/param/admin/kind_box"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	richerror "git.gocasts.ir/ebhomengo/niki/pkg/rich_error"
	validation "github.com/go-ozzo/ozzo-validation/v4"
)

func (v Validator) ValidateGetRequest(req param.KindBoxGetRequest) (map[string]string, error) {
	const op = "adminkindboxvalidator.ValidateGetRequest"

	if err := validation.ValidateStruct(&req,
		validation.Field(&req.BenefactorID,
			validation.Required,
			validation.By(v.doesBenefactorExist)),

		validation.Field(&req.KindBoxID,
			validation.Required,
			validation.By(v.doesKindBoxBelongToBenefactor(req.BenefactorID)),
			validation.By(v.doesKindBoxExist)),
	); err != nil {
		fieldErrors := make(map[string]string)

		var errV validation.Errors
		if errors.As(err, &errV) {
			for key, value := range errV {
				if value != nil {
					fieldErrors[key] = value.Error()
				}
			}
		}

		return fieldErrors, richerror.New(op).
			WithMessage(errmsg.ErrorMsgInvalidInput).
			WithKind(richerror.KindInvalid).
			WithMeta(map[string]interface{}{"req": req}).
			WithErr(err)
	}

	return map[string]string{}, nil
}
